# ADDLOG.CL -- CL script that takes an imagename as an argument.
# It appends a record to the current observing log file.
# January 16, 2007
# By Doug Mink
 
procedure addlog (image)
 
string  image   	{prompt="Images to be logged"}
bool	verbose=yes	{prompt="Print error messages (yes or no)?"}

string	*list

begin
	string  imglist, img, img0, tmp, tmplist, logfile
	string	arguments, program
	string	logroot
	string  instname=""
	bool	logexists
	string	argv

	program = osfn ("obslog$bin(arch)/add_log")

# Check for existence of log file
	logroot = obslog.logroot
	if (strlen (logroot) <= 0) {
	    if (verbose)
		print ("Log file root name not defined - epar obslog")
	    return
	    }
	else {
	    logfile = obslog.logdir // logroot // ".lis"
	    }

# Set instrument	
	instname = obslog.instname
	if (strlen (instname) <= 0) {
	    if (verbose)
		printf ("Instrument name not defined\n")
	    return
	    }
	if (verbose)
	    argv = "-v "
	else
	    argv = ""
 
# Read through list of images to add to log
        imglist = image
	tmplist = mktemp ("tmp$log.")
	sections (imglist, option="fullname", > tmplist)
	list = tmplist
	logexists = access ("loghead.tex") && access ("logpage.tex")
	while (fscan (list, img0) != EOF) {

# If log listing file not here yet, write log header files 
	    if (!logexists) {
		initlog (img0)
		logexists = yes
		}

#	    if (verbose)
#		printf ("ADDLOG: processing %s image %s\n", instname,img0)
	
	    if (instname == "4sh") {
		arguments = argv // instname // " " // osfn (img0) // ",1"
		printf ("!%s %s >> %s\n", program,
			arguments, osfn(logfile)) | cl
		}
	    else if (instname == "1sh"){
		arguments = argv // instname // " " // osfn (img0)
		printf ("!%s %s >> %s\n", program,
			arguments, osfn(logfile)) | cl
		}
	    else if (instname == "ech") {
		arguments = argv // instname // " " // osfn (img0)
		printf ("!%s %s >> %s\n", program,
			arguments, osfn(logfile)) | cl
		}

	    else if (instname == "fast") {
		arguments = argv // instname // " " // osfn (img0)
		printf ("!%s %s >> %s\n", program,
			arguments, osfn(logfile)) | cl
		}
	    else
		printf ("Unknown instrument name for logging.\n")
	}

	list = ""
	delete (tmplist, verify-, >& "dev$null")
end

# Apr 19 2001	(Doug Mink) Use addfast for FAST spectrum logging
# Apr 20 2001	(Doug Mink) Increment pagenumber here
# May  2 2001	(Doug Mink) Use add_ccd for 1- and 4-shooter
# May  9 2001	(Doug Mink) Use add_log for everything
# May 10 2001	(Doug Mink) Convert to obslog package
#
# Nov 17 2005	(Doug Mink) Use bin(arch) instead of bin; add program variable
#
# Jan 16 2007	(Doug Mink) Pass verbose to foreign task add_log
