/* Program to extract echelle spectrum data for logging
 * This routine reads parameters from an image header and writes
 * strings that can be picked up for the master log program

 * originally by Todd R. Lauer	4/9/94
 * Modified to read from image file by Doug Mink, 2001-05-09
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "fitsfile.h"
#include "wcstab.h"

#define MAXC 640		/* Length of input line.		*/
#define MAXW 640		/* Maximum word length.			*/

int
add_ech (imagefile, verbose)

char	*imagefile;
int	verbose;
{
	int	hr, mn, sec;
	int	lw;
	int	debug;
	float 	rsec;
	double	ra, dec;
	char	word[MAXW],
		texw[MAXW],
		texw2[MAXW],
		texw3[MAXW],
		line[MAXC],
		*endword,
		*header,
		*GetFITShead();

	debug = 0;

	/* Read the image file header */
	if ((header = GetFITShead (imagefile)) == NULL) {
            fprintf (stderr, "ADD_ECH: Cannot read image header\n");
            return (-1);
            }
	else if (debug)
		fprintf (stderr, "ADD_ECH: image file %s\n", imagefile);
 
	/* Delete the extension from the filename and write to start of line */
	strcpy (word, imagefile);
	lw = strlen (word);
	endword = word + lw - 1;
	while (*endword != '.')
	    *endword = (char) 0;
	*endword = (char) 0;
	sprintf (texw,"%.20s", word);
	texcheck(texw);
	strcpy (line, texw);
	if (debug)
	    fprintf (stderr, "ADD_ECH: image %s\n",texw);

	/* Exposure time to nearest second */
	hgetr4 (header, "EXPTIME", &rsec);
 	sprintf (texw," %.0f", rsec);
	strcat (line, " ");
	strcat (line, texw);
	if (debug)
	    fprintf (stderr, "ADD_ECH: exposure time = %s\n",texw);

	/* Round the UT start time to the nearest minute */
	hgets (header, "UT", MAXW, word);
	sscanf (word,"%2d:%2d:%2d",&hr,&mn,&sec);
	if (sec >= 30)
	    mn++;
	if (mn >= 60) {
	    mn -= 60;
	    hr++;
	    }
	if (hr >= 24)
	    hr -= 24;
	if (mn >= 10)
	    sprintf (texw," %2d:%2d", hr, mn);
	else
	    sprintf (texw," %2d:0%1d", hr, mn);
	strcat (line, " ");
	strcat (line, texw);
	if (debug) fprintf (stderr, "ADD_ECH: U.T. = %s\n",texw);

	/* Right ascension */
	hgets (header, "RRA", MAXW, word);
	ra = str2dec (word);
	ra2str (texw2, MAXW, ra, 0);
	sprintf (texw," %s", texw2); 
	strcat (line, " ");
	strcat (line, texw);
	if (debug)
	    fprintf (stderr, "ADD_ECH: RA = %s\n",texw);

	/* Declination */
	hgets (header, "RDEC", MAXW, word);
	dec = str2dec (word);
	dec2str (texw3, MAXW, dec, 0);
        sprintf (texw," %s", texw3); 
	strcat (line, " ");
	strcat (line, texw);
	if (debug)
	    fprintf (stderr, "ADD_ECH: Dec = %s\n",texw);

	/* Program */
	strcpy (word, "-");
	hgets (header, "PROGRAM", MAXW, word);
	sprintf (texw," \"%.14s\"", word);
	texcheck(texw);   
	strcat (line, " ");
	strcat (line, texw);
	if (debug)
	    fprintf (stderr, "ADD_ECH: Program = %s\n",texw);

	/* Read observation comments from REMARK keyword */
	strcpy (word, " ");
	hgets (header, "REMARK", MAXW, word);
	sprintf (texw," \"%s\"", word);
	texcheck(texw);   
	strcat (line, " ");
	strcat (line, texw);

	/* Print completed line to standard output */
	printf ("%s\n", line); 
	if (verbose)
	    fprintf (stderr, "%s\n", line); 
	return (0);
}
/* Jan 16 2007	Print output line to stderr if verbose mode
 */
