/* tex_ech.c
 * Write TeX file from ASCII Echelle log, returning number of pages created
 * Latest revision by Doug Mink, January 12, 2006
 * Revised by Nelson Caldwell 3/1999
 * By Tod R. Lauer 9/23/1996
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <ctype.h>
#include <string.h>
#include "fitsfile.h"
#include "wcstab.h"

#define MAXC 640		/* Length of input line. */
#define MAXW 640		/* Maximum word length.	 */
#define MAXB 5000		/* Maximum buffer length */

int
tex_ech (logroot, libdir)

char	*logroot;	/* Rootname of log file */
char	*libdir;	/* Pathname of obslog lib directory */

{
	int	i, nline, npage;
	int	ntok, nlog, ntok0, nsame;

	FILE	*log, *tex;

	struct Tokens tokens;	/* Token structure returned */
	double dec, dec0;
	int	last = 0;
	char	sname[MAXW],	/* individual parameters from log file */
		snum[MAXW],
		sobj[MAXW],
		sexp[MAXW],
		sst[MAXW],
		sra[MAXW],
		sdec[MAXW],
		sprog[MAXW],
		scoma[MAXW],
		scomb[MAXW];
	char	sname0[MAXW],
		snum0[MAXW],
		sobj0[MAXW],
		sexp0[MAXW],
		sst0[MAXW],
		sra0[MAXW],
		sdec0[MAXW],
		sprog0[MAXW],
		scoma0[MAXW],
		scomb0[MAXW];
	char	inline1[MAXC],
		snum1[MAXW],
		word[MAXW],
		texw[MAXW],
		file[MAXC],
	        file2[MAXC],
	        comment_2[MAXC];
	char	outline[MAXB];
	char	*oline, *olast;
	char	czero = (char) 0;
	char	*lp;

	/* fprintf (stderr, "TEX_FAST: creating %s.tex\n", logroot); */

	/* Create TeX output file */
	strcpy (file, logroot);
	strcat (file, ".tex");
	if (open (file, O_WRONLY | O_CREAT | O_TRUNC, 0644) == -1) {
	    printf ("Error creating TeX file %s\n", file);
	    return (0);
	    }
 
	if ((tex = fopen (file,"w")) == NULL) { 
	    printf ("Error opening TeX file %s\n", file); 
	    return (0);
	    } 
 
	/* Put in standard header stuff and first page number */
	strcpy (file, libdir);
	strcat (file, "/");
	strcat (file, "kptab1ech");
	fprintf (tex, "\\input %s\n", file);
	fprintf (tex, "\\input logpage\n");
	fprintf (tex, "\\offinterlineskip\n");

	/* Now open the logfile */
	strcpy (file2, logroot);
	strcat (file2, ".lis");
	if ((log = fopen (file2,"r")) == NULL) {
	    fprintf (tex, "\\noindent Error opening main logfile %s\n", file2);
	    fprintf (tex, " --- No observations recorded yet?\\par\n");
	    fprintf (tex, "\\vfill}\\end\n");
	    return (0);
	    }
 
	/* Get lines until end of log file is reached */
	nline	= 0;
	nlog = 0;
	nsame = 0;
	npage = 0;
	last = 0;
	while ((lp = fgets (inline1, MAXC, log)) != NULL) {

	    /* Get info for this line */
	    if (!last) {
		nlog++;

		/* Parse next line from log file */
		setoken (&tokens, inline1, "");
		ntok = tokens.ntok;

		/* File name */
		(void) getoken (&tokens, 1, sname, MAXC);
		strcpy (sobj, sname+5);
		strncpy (snum, sname, 4);
		snum[4] = (char) 0;

		(void) getoken (&tokens, 2, sexp, MAXC);	/* Exposure time */
		(void) getoken (&tokens, 3, sst, MAXC);	/* UT Start time */
		(void) getoken (&tokens, 4, sra, MAXC);	/* Right Ascension */
		(void) getoken (&tokens, 5, sdec, MAXC);	/* Declination */
		dec = str2dec (sdec);
		(void) getoken (&tokens, 6, sprog, MAXC); /* Program Name */
		if (ntok > 6)				/* Remarks */
		    (void) getoken (&tokens, 7, scoma, MAXC);
		else
		    strcpy (scoma, " ");
		if (ntok > 7)
		    (void) getoken (&tokens, 8, scomb, MAXC);
		else
		    strcpy (scomb, " ");
		}

	    /* First time through, just save parameters */
	    if (nlog == 1) {
		strcpy (sname0, sname);
		strcpy (snum0, snum);
		strcpy (sobj0, sobj);
		strcpy (sexp0, sexp);
		strcpy (sst0, sst);
		strcpy (sra0, sra);
		strcpy (sdec0, sdec);
		strcpy (sprog0, sprog);
		strcpy (scoma0, scoma);
		strcpy (scomb0, scomb);
		dec0 = dec;
		ntok0 = ntok;
		continue;
		}

	    /* Compare to previous line */
	    else if ((!strcmp (sobj, "DARK") || !strcmp (sobj, "BIAS") ||
		 !strcmp (sobj, "FLAT") || !strcmp (sobj, "SKY") ||
		 !strcmp (sobj, "sky") || !strcmp (sobj, "Sky")) &&
		 dec == dec0 &&
		(!strcmp (sobj, sobj0) && !strcmp (sexp, sexp0) &&
		 !strcmp (scoma, scoma0) && !strcmp (scomb, scomb0))) {
		nsame++;
		if (nsame == 1)
		    strcpy (snum1,snum0);
		strcpy (sname0, sname);
		strcpy (snum0, snum);
		strcpy (sobj0, sobj);
		strcpy (sexp0, sexp);
		strcpy (sst0, sst);
		strcpy (sra0, sra);
		strcpy (sdec0, sdec);
		strcpy (sprog0, sprog);
		strcpy (scoma0, scoma);
		strcpy (scomb0, scomb);
		dec0 = dec;
		ntok0 = ntok;
		if (!last) {
		    if (lp == NULL)	/* If last line, set flag */
			last = 1;
		    continue;
		    }
		}
	    if (nsame > 0)
		sprintf (sname0, "%4s-%4s.%s",snum1, snum0, sobj0);

	    nsame = 0;

	    /* Finish page after 20th line */
	    if (nline == 20) {
		fprintf (tex, "}\\vfill}\\eject\n");
		nline	= 0;
		}

	    /* Print page header on first line */
	    if (nline == 0) {
	        fprintf (tex, "\\vbox{\\null\\vskip 3.7pt\n");
		fprintf (tex, "\\input loghead\n");
	        fprintf (tex, "\\vskip 7pt\n");
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "sao.logo.ps");
		fprintf (tex,
		    "\\special{psfile=%s hoffset=-253 voffset=-340}\n",file);
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "spec.eps");
		fprintf (tex, "\\special{psfile=%s hoffset=453 voffset=-60}\n",file);
		strcpy (file, libdir);
		strcat (file, "/");
		strcat (file, "kptab2ech");
		fprintf (tex, "\\input %s\n", file);
		npage++;
		}

	    /* Write output TeX line */
	    nline++;
	    olast = outline + MAXB;
	    for (oline = outline; oline < olast; oline++)
		*oline = czero;

	    /* File name */
	    texlong (sname0, 19); 
	    sprintf (texw,"\\ %s&", sname0);
	    strcpy (outline, texw);

	    /* Exposure time */
	    sprintf (texw,"%s\\thinspace&", sexp0);
	    strcat (outline, texw);

	    /* UT Start time */
	    sprintf (texw,"%s\\thinspace&", sst0);
	    strcat (outline, texw);

	    /* Right Ascension */
	    sprintf (texw,"%s&", sra0);
	    strcat (outline, texw);

	    /* Declination */
	    sprintf (texw,"%s&", sdec0);
	    strcat (outline, texw);

	    /* Program Name */
	    texlong (sprog0, 18);
	    sprintf (texw,"%s&", sprog0);
	    strcat (outline, texw);

	    /* Remarks */
	    if (ntok0 > 10) {
		if (ntok > 11) {
		    strcpy (comment_2, word);
		    strcpy (word, scoma0);
		    strcat (word, " ");
		    strcat (word, scomb0);
		    texlong (word, 26);
		    sprintf (texw,"\\thinspace %s&\\cr", word);
		    strcat (outline, texw);
		    }
		else {
		    strcpy (word, scoma0);
		    texlong (word, 26);
		    sprintf (texw,"\\thinspace %s&\\cr", word);
		    strcat (outline, texw);
		    }
		}

/*      Check to make sure that we got all the words.  If we didn't,
	and error on input has occured, and the line must be completed  */
 
	    if (ntok0 < 11) {
		for (i = ntok; i < 12; i++)
		    strcat (outline," &");
		strcat (outline, "\\cr");
	        }

	    /* Print this line of the log */
	    fprintf (tex, "%s\n", outline);
	    fprintf (tex, "\\noalign{\\khrule}\n");

	    if (last)
		break;

	    /* If last line, set flag */
	    if (lp == NULL)
		last = 1;

	    /* Save latest values */
	    strcpy (sname0, sname);
	    strcpy (snum0, snum);
	    strcpy (sobj0, sobj);
	    strcpy (sexp0, sexp);
	    strcpy (sra0, sra);
	    strcpy (sdec0, sdec);
	    strcpy (sprog0, sprog);
	    strcpy (scoma0, scoma);
	    strcpy (scomb0, scomb);
	    dec0 = dec;
	    inline1[0] = (char) 0;
	    }

	fprintf (tex, "}\\vfill}\\end\n");
	(void) fclose (tex);
	(void) fclose (log);
	return (npage);
}
/* May 30 2001	Revised by Doug Mink
 * Jul 11 2001	Output line buffer size increased to deal with long comments
 *
 * Feb 14 2002	Add maximum length to getoken() calls
 *
 * Nov 15 2005  Set last line if null string read, not from file structure
 *
 * Jan 11 2007	Add more include files; change creat() to open()
 * Jan 12 2007	Pass pointer to token string structure to getoken()
 */
