/*	Routine to check for long TeX words				*/

/*	Author:	Tod R. Lauer	2/6/94
	Modified by Doug Mink, December 12, 2005	*/

/*	This routine checks lines shipped out to TeX that might be too
	long for their boxes.  If so, the word is rebuilt with smaller
	fonts, or as double lines in a box if needed.			*/

#include <stdio.h>
#include <string.h>
#define MAXC 640

int
texlong (word, max)

char *word;	/* TeX input						*/
int max;	/* Normal number of maximum characters			*/

{
	char	line[MAXC], hold[MAXC], *wp;
	int	i, nb, nc, nl, math, backslash;
	float	cm;

	wp	= word;

	/* First, just count the number of characters */
	math	= 0;
	backslash = 0;
	nc = 0;
	for (i = 0; *wp != (char) 0; i++) {
	    if (*wp == '$') {
		if (backslash) {
		    nc++;
		    backslash = 0;
		    }
		math	= 1 - math;
		}
	    else if (*wp == '\\') {
		if (backslash)
		    nc++;
		else
		    backslash = 1;
		}
	    else if (backslash) {
		if (*wp == ' ' || *wp == '{' || *wp == '}' ||
		    *wp == '(' || *wp == ')' ||
		    *wp == '_' || *wp == '^') {
		    nc++;
		    backslash	= 0;
		    }
		else if ((*wp >= '0' && *wp <= '9') ||
			  *wp == '+' || *wp == '-' || *wp == '/') {
		    nc	+= 2;
		    backslash	= 0;
			}
		}
	    else if (math && *wp != ' ')
		nc++;
	    else 
		nc++;
	    wp++;
	    }

	/* If less than max, just return */
	if (nc < max)
		return(0);

	/* Test to see if a simple font change will work */
	if (nc < max * 44 / 40) {
	    strcpy (line,"{\\kpnosm\\ksmtt ");
	    strcat (line,word);
	    strcat (line,"}");
 	    }
	else if (nc < max * 53 / 40) {
	    strcpy (line,"{\\kpnosmr\\ksmttt ");
	    strcat (line,word);
	    strcat (line,"}");
 	    }
	else if (nc <= max * 59 / 40 + 1) {
	    strcpy (line,"{\\kpnosmst\\ksmtttt ");
	    strcat (line,word);
	    strcat (line,"}");
	    }
	else {

	    /* Double the line for really long words */
	    nl	= max * 59 / 40;

	    /* Look for the last blank in a long word */
	    nb = 0;
	    math = 0;
	    wp = word;
	    for (i = 0; i < nl; i++) {
		if (*wp == ' ' && !math)
		    nb	= i;
		if (*wp == '$')
		    math = 1 - math;
		wp++;
		}

	    wp = word;
	    cm = 8.9 * max / 40.0;
	    strcpy (line, "\\vbox to 15pt{\\parindent=0pt\\vskip 1.6pt");
	    sprintf (hold, "{\\hbox to %3.1fcm{\\kpnosmst\\ksmtttt ", cm);
	    strcat (line, hold);
	    if (nb == 0) {
		strncat(line, wp, nl);
		wp	+= nl;
		}
	    else {
		strncat(line, wp, nb);
		wp	+= nb + 1;
		sprintf (hold, "\\hfil}\\vskip 1.6pt\\hbox to %3.1fcm{\\kpnosmst\\ksmtttt ", cm);
		strcat (line, hold);
		strcat (line, wp);
		}

	    strcat (line, "\\hfil}}}");
	    }

	strcpy (word,line);
	line[0] = (char) 0;

	return (0);
}
/* Apr 19 2001	(Doug Mink) clean up code
 *
 * Nov 15 2005	Compare to (char) 0, not NULL
 * Dec 12 2005	If no blanks, truncate line to avoid problems
 *
 * Jan 12 2007	Declare int; add final return
 */
