#ifndef _wcstab_h_
#define _wcstab_h_

/* Data structure for tab table files */
struct TabTable {
    char *filename;     /* Name of tab table file */
    int nlines;         /* Number of entries in table */
    char *tabname;      /* Name of this table or NULL */
    char *tabbuff;      /* Pointer to start of saved tab table in memory */
    char *tabheader;    /* Pointer to start of line containing table header */
    char *tabhead;      /* Pointer to start of line containing column heading */
    char *tabdash;      /* Pointer to start of line with dashes after column headings */
    char *tabdata;      /* Pointer to start of first line of table data */
    int lhead;          /* Number of bytes before first data line */
    int iline;          /* Number of current line (1=first) */
    int lline;          /* Length in bytes of line buffer */
    char *tabline;      /* Pointer to start of current line */
    FILE *tcat;         /* File descriptor for tab table file */
    int ncols;          /* Number of columns per table entry */
    char **colname;     /* Column names */
    int *lcol;          /* Lengths of column header names */
    int *lcfld;         /* Number of columns in field (hyphens) */
    int lbuff;          /* Number of bytes in entire tab table */
};

/* Subroutines for extracting tab table information */

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __STDC__   /* Full ANSI prototypes */

    struct TabTable *tabopen(   /* Open tab table file */
	char *tabfile,  /* Tab table catalog file name */
	int nbbuff);    /* Number of bytes in buffer; 0=read whole file */
    void tabclose(      /* Free all arrays left open by tab table structure */
	struct TabTable *tabtable); /* Tab table data structure */
    char *gettabline(   /* Find a specified line in a tab table */
	struct TabTable *tabtable, /* Tab table data structure */
	int iline);     /* Line sequence number in tab table */
    double tabgetra(    /* Return right ascension in degrees from tab table*/
	struct Tokens *tabtok,  /* Line token structure */
	int ientry);    /* sequence of entry on line */
    double tabgetdec(   /* Return declination in degrees from tab table*/
	struct Tokens *tabtok,  /* Line token structure */
	int ientry);    /* sequence of entry on line */
    double tabgetr8(    /* Return double number from tab table line */
	struct Tokens *tabtok,  /* Line token structure */
	int ientry);    /* sequence of entry on line */
    int tabgeti4(       /* Return 4-byte integer from tab table line */
	struct Tokens *tabtok,  /* Line token structure */
	int ientry);    /* sequence of entry on line */
    void settabkey(     /* Set tab table keyword to read for object */
	char *keyword); /* column header of desired value */
    int tabgetk(        /* Get tab table entries for named column */
	struct TabTable *tabtable, /* Tab table data structure */
	struct Tokens *tabtok,  /* Line token structure */
	char *keyword,  /* column header of desired value */
	char *string,   /* character string (returned) */
	int maxchar);   /* Maximum number of characters in returned string */
    int tabgetc(        /* Get tab table entry for named column */
	struct Tokens *tabtok,  /* Line token structure */
	int ientry,     /* sequence of entry on line */
	char *string,   /* character string (returned) */
	int maxchar);   /* Maximum number of characters in returned string */
    int tabparse(               /* Aeturn column names and positions in tabtable */
	struct TabTable *tabtable); /* Tab table data structure */
    int tabcol(         /* Find column for name */
	struct TabTable *tabtable, /* Tab table data structure */
	char *keyword); /* column header of desired value */
    int istab(          /* Return 1 if tab table file, else 0 */
	char *filename); /* Name of file to check */
    char *gettaberr();  /* Return most recent tab table error message */
    int gettabndec();   /* Return number of decimal places in tab catalog ids */

    int texlong (	/* Check for long TeX words */
	char *word,	/* TeX input */
	int max);	/* Normal number of maximum characters */
    void texcheck(	/* Escape TeX special characters #$%^&_  w/ backslash */
	char *word);
    void texclear(	/* Clear a string buffer */
	char *str,	/* String to clear */
	int nbytes);	/* Number of bytes to clear */

#else /* K&R prototypes */

    struct TabTable *tabopen(); /* Open tab table file */
    char *gettabline();	/* Find a specified line in a tab table */
    int tabrkey();	/* Keyword values from tab table catalogs */
    int tabcol();	/* Find column for name */
    int tabgetk();	/* Get tab table entries for named column */
    int tabgetc();	/* Get tab table entry for named column */
    int tabgeti4();	/* Return 4-byte integer from tab table line */
    int tabparse();	/* Aeturn column names and positions in tabtable */
    double tabgetra();	/* Return right ascension in degrees from tab table*/
    double tabgetdec();	/* Return declination in degrees from tab table*/
    double tabgetpm();	/* Return RA or Dec p.m. in degrees from tab table*/
    double tabgetr8();	/* Return double number from tab table line */
    void tabclose();	/* Free all arrays left open by tab table structure */
    char *gettaberr();	/* Return most recent tab table error message */
    int istab();
    int gettabndec();	/* Return number of decimal places in tab catalog ids */

    int texlong ();	/* Check for long TeX words */
    void texcheck();	/* Escape special TeX characters #$%^&_  w/ backslash */
    void texclear();	/* Clear a string buffer */

#endif  /* __STDC__ */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif  /* _wcstab_h_ */
